package XXV::MODULES::EPG;
use strict;

use Tools;
use File::Basename;
use Locale::gettext;


# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'EPG',
        Prereq => {
            'Date::Manip' => 'date manipulation routines',
            'Time::Local' => 'efficiently compute time from local and GMT time ',
        },
        Description => gettext('This module reads new EPG Data and stores them in the database.'),
        Version => '0.91',
        Date => '23.08.2006',
        Author => 'xpix',
        Status => sub{ $obj->status(@_) },
        Preferences => {
            epgimages => {
                description => gettext('Location of additional EPG images.'),
                default     => '/var/cache/vdr/epgimages',
                type        => 'dir',
            },
            interval => {
                description => gettext('The interval to parse new EPG to DB in sec'),
                default     => 600,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
            periods => {
                description => gettext("Pre-defined list for the 'Running Now' view (comma separated list)"),
                default     => '12:00,18:00,20:15,22:00',
                type        => 'string',
                required    => gettext('This is required!'),
            },
            timeframe => {
                description => gettext("How much hours to display in schema"),
                default     => 2,
                type        => 'integer',
                required    => gettext('This is required!'),
            },
        },
        Commands => {
            search => {
                description => gettext('Search in EPG Data'),
                short       => 's',
                callback    => sub{ $obj->search(@_) },
            },
            program => {
                description => gettext("List program for channel 'channel name'"),
                short       => 'p',
                callback    => sub{ $obj->program(@_) },
            },
            display => {
                description => gettext("Show program event 'eventid'"),
                short       => 'd',
                callback    => sub{ $obj->display(@_) },
            },
            now => {
                description => gettext('Display events is running now'),
                short       => 'n',
                callback    => sub{ $obj->runningNow(@_) },
            },
            next => {
                description => gettext('Display events is running next'),
                short       => 'nx',
                callback    => sub{ $obj->runningNext(@_) },
            },
            schema => {
                description => gettext('Display events in a schematic way'),
                short       => 'sch',
                callback    => sub{ $obj->schema(@_) },
            },
            erestart => {
                description => gettext('Reload EPG data'),
                short       => 'er',
                callback    => sub{
                    my $watcher = shift || return error ('No Watcher!');
                    my $console = shift || return error ('No Console');

                    lg sprintf('Start reload EPG data by user: %s',
                        $console->{USER}->{Name}
                        );

                    $obj->startReadEpgData($watcher,$console);
                },
                Level       => 'admin',
            },
            erun => {
                description => gettext('Display the epg event running in vdr'),
                short       => 'en',
                callback    => sub{ $obj->NowOnChannel(@_) },
            },
            conflict => {
                hidden      => 'yes',
                callback    => sub{ return $obj->checkOnTimer(@_) },
            },
        },
    };
    return $args;
}

# ------------------
sub status {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;
    my $lastReportTime = shift || 0;

    my $sql = "select count(*) from EPG";
    my $gesamt = $obj->{dbh}->selectall_arrayref($sql)->[0][0];

    $sql = "select count(*) from EPG where UNIX_TIMESTAMP(addtime) > $lastReportTime";
    my $newEntrys = $obj->{dbh}->selectall_arrayref($sql)->[0][0];

    return {
        message => sprintf(gettext('EPG table contains %d entries and since the last login at %s %d new entries'),
            $gesamt, scalar localtime($lastReportTime), $newEntrys),
    };
}

# ------------------
sub new {
# ------------------
    my($class, %attr) = @_;
    my $self = {};
    bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

    # who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_};
        $self->{$_} = $self->{MOD}->{Preferences}->{$_}->{default} unless($self->{$_});
    } keys %{$self->{MOD}->{Preferences}};

    # Try to use the Requirments
    map {
        eval "use $_";
        return panic("\nCan not load Module: $_\nPlease install this module on your System:\nperl -MCPAN -e 'install $_'") if($@);
    } keys %{$self->{MOD}->{Prereq}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # The Initprocess
    $self->_init or return error('Problem to initialze Module');

    return $self;
}

# ------------------
sub _init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    $obj->{LastEpgChangedTime} = 0;

    return 0, panic("Session to database is'nt connected")
      unless($obj->{dbh});

    # Look for table or create this table
    foreach my $table (qw/EPG OLDEPG/) {

      # remove old Version, if updated
      if(tableExists($obj->{dbh}, $table)) {
          my $fields = fields($obj->{dbh}, 'select * from '.$table);
          unless(scalar @$fields == 14) {
              $obj->{dbh}->do('drop table '.$table);
          }
      }

        my $version = main::getVersion;
        $obj->{dbh}->do(qq|
          CREATE TABLE IF NOT EXISTS $table (
              eventid bigint unsigned NOT NULL default '0',
              title text NOT NULL default '',
              subtitle text default '',
              description text,
              channel_id varchar(100) NOT NULL default '',
              starttime datetime NOT NULL default '0000-00-00 00:00:00',
              duration int(11) NOT NULL default '0',
              tableid tinyint(4) default 0,
              image text default '',
              version tinyint(3) default 0,
              video varchar(100) default '',
              audio varchar(255) default '',
              addtime datetime NOT NULL default '0000-00-00 00:00:00',
              vpstime datetime default '0000-00-00 00:00:00',
              PRIMARY KEY (eventid),
              INDEX (starttime),
              INDEX (channel_id)
            ) COMMENT = '$version'
        |);
    }

    # Repair later Data ...
    main::after(sub{
        $obj->startReadEpgData();

        # Restart watcher every x hours
        Event->timer(
            interval => $obj->{interval},
            prio => 6,  # -1 very hard ... 6 very low
            cb => sub{
                lg sprintf('The read on epg data is restarted!');
                $obj->startReadEpgData();
            },
        );
    }, "EPG: Start read epg data and repair ...", 40);

    return 1;
}

# ------------------
sub startReadEpgData {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;

    my $waiter;
    if(ref $console && $console->typ eq 'HTML') {
        $waiter = $console->wait(gettext("Read EPG data ..."),0,1000,'no');
    }

    # Read data over SVDRP
    my $smod = main::getModule('SVDRP') 
      or return error('Can not get for SVDRP Module');
    my $vdata = $smod->command('LSTE','|');
    map { 
      $_ =~ s/^\d{3}.//;
      $_ =~ s/[\r|\n]$//;
    } @$vdata;
    debug sprintf('The read on epg data start now!');


    # Adjust waiter max value now.
    $waiter->max(scalar @$vdata)
        if(ref $console && ref $waiter);

    $obj->moveOldEPGEntrys();

    # Read file row by row
    my $updated = $obj->compareEpgData($vdata,$watcher,$console,$waiter);

    $obj->deleteDoubleEPGEntrys();

    # last call of waiter
    $waiter->end() if(ref $waiter);

    if(ref $console) {
        $console->start() if(ref $waiter);
        $console->message(sprintf(gettext("%d events in database updated."), $updated));

        $console->redirect({url => $console->{browser}->{Referer}, wait => 2})
            if($console->typ eq 'HTML');
    }
}

# This Routine will compare data from epg.data
# and EPG Database row by row
# ------------------
sub compareEpgData {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $vdata = shift || return error('No data');
    my $watcher = shift;
    my $console = shift;
    my $waiter = shift;

    my $changedData = 0;
    my $updatedData = 0;
    my $deleteData = 0;

    # Second - read data
    my $count = 0;

    my $vdrData;
    my $channel;
    my $channelname;
    while($count < scalar $vdata) {
      ($vdrData,$channel,$channelname,$count) = $obj->readEpgData($vdata,$count);
      last if(not $channel);

      $waiter->next($count,undef, sprintf(gettext("Analyze channel '%s'"), $channelname))
        if(ref $waiter);

      # First - read database
      my $sql = qq|select eventid, title, subtitle, length(description) as ldescription, duration, UNIX_TIMESTAMP(starttime) as starttime, UNIX_TIMESTAMP(vpstime) as vpstime, video, audio from EPG where channel_id = '$channel' |;
      my $db_data = $obj->{dbh}->selectall_hashref($sql, 'eventid');

      lg sprintf( 'Compare EPG Database with data from VDR Data : %d / %d for channel %s', scalar keys %$db_data,scalar keys %$vdrData, $channel);
      # Compare this Hashes
      foreach my $eid (keys %{$vdrData}) {
        my $row = $vdrData->{$eid};

        # Not exists in DB .. insert
        if(exists $db_data->{$eid}) {
          # Compare fields
          foreach my $field (qw/title subtitle ldescription duration starttime vpstime video audio/) {
            next if(not exists $row->{$field} or not $row->{$field});
            if((not exists $db_data->{$eid}->{$field})
                or (not $db_data->{$eid}->{$field})
                or ($db_data->{$eid}->{$field} ne $row->{$field})) {
              $obj->replace($eid, $row);
              $updatedData++;
              last;
            }
          }
        } else {
          $obj->replace($eid, $row);
          $changedData++;
        }
        # Delete all rows to discover the 
        # rows to delete in DB
        delete $db_data->{$eid};
      }

      # Delete unused EpgEntrys in DB 
      $deleteData += scalar keys %$db_data;
      if(scalar keys %$db_data > 0) {
        my $sql = sprintf('delete from EPG where eventid in (%s)', join(', ', keys %$db_data));
        $obj->{dbh}->do($sql);
      }
    } 
    debug 'Finish .. %d events created, %d events replaced, %d events deleted', $changedData, $updatedData, $deleteData;

    # Set Timeflag, if data changed
    $obj->{LastEpgChangedTime} = time
      if($changedData);

    return ($changedData + $updatedData + $deleteData);
}

# ------------------
sub moveOldEPGEntrys {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # Copy and delete old EPG Entrys
    $obj->{dbh}->do('REPLACE INTO OLDEPG SELECT * FROM EPG WHERE (UNIX_TIMESTAMP(EPG.starttime) + EPG.duration) < UNIX_TIMESTAMP()');
    $obj->{dbh}->do('DELETE FROM EPG WHERE (UNIX_TIMESTAMP(EPG.starttime) + EPG.duration) < UNIX_TIMESTAMP()');
}

# ------------------
sub deleteDoubleEPGEntrys {
# ------------------
    my $obj = shift || return error ('No Object!' );

    # Delete double EPG Entrys
    my $erg = $obj->{dbh}->selectall_arrayref('SELECT eventid FROM EPG GROUP BY starttime, channel_id having count(*) > 1');
    lg sprintf('Repair data found %d wrong events!', scalar @$erg)
      if(scalar @$erg > 0);
    foreach my $row (@$erg) {
        $obj->{dbh}->do(sprintf("DELETE FROM EPG WHERE eventid = '%lu'", $row->[0]));
    }
}

# ------------------
sub replace {
# ------------------
    my $obj = shift || return error ('No Object!');
    my $eventid = shift || return error ('No eventid to insert!');;
    my $attr = shift || return error ('No data to insert!');

    my $sth = $obj->{dbh}->prepare('REPLACE INTO EPG(eventid, title, subtitle, description, channel_id, duration, tableid, image, version, video, audio, starttime, addtime, vpstime) VALUES (?,?,?,?,?,?,?,?,?,?,?,FROM_UNIXTIME(?),FROM_UNIXTIME(?),FROM_UNIXTIME(?))');
    $sth->execute(
        $eventid,
        $attr->{title},
        $attr->{subtitle},
        $attr->{description},
        $attr->{channel},
        $attr->{duration},
        $attr->{tableid},
        $attr->{image} || '',
        hex($attr->{version}),
        $attr->{video} || '1 01 deu 4:3',
        $attr->{audio} || "2 03 deu stereo",
        $attr->{starttime},
        time,
        $attr->{vpstime}
    ) if($attr->{channel});
}

# ------------------
sub encodeEpgId {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $epgid = shift || return error('No EPG Id!');
    my $channel = shift || return error('No Channel!');

    # look for NID-TID-SID for unique eventids (SID 0-30000 / TID 0 - 1000 / NID 0 - 10000
    my @id = split('-', $channel);

    # Make a fix format 0xCCCCEEEE : C-Channelid (high-word), E-Eventid(low-word) => real-eventid = uniqueid & FFFF
    my $eventid = ((($id[-3] + $id[-2] + $id[-1]) & 0x3FFF) << 16) | ($epgid & 0xFFFF);

    return $eventid;
}

# ------------------
sub readEpgData {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $vdata = shift || return error('Problem to read Data!');
    my $count = shift || 0;
    my $dataHash = {};

    my $cmod = main::getModule ('CHANNELS');
    my $channels = $cmod->ChannelArray ('Id,Name');
    my $channel;
    my $channelname;
    my $event;

    #debug 'Read %d lines in EPG Database from %d', scalar @$vdata,$offset;
    for(;$count < scalar (@$vdata);$count++) {
      my $line = @{$vdata}[$count];

      # Ok, Datarow complete...
      if($line eq 'e' and $event->{eventid} and $event->{channel}) {
        if(-e sprintf('%s/%d.png', $obj->{epgimages}, $event->{eventid})) {
          my $firstimage = sprintf('%d.png',$event->{eventid});
          $event->{image} = $firstimage."\n";
          my $imgpath = sprintf('%s/%d_?.png',$obj->{epgimages},$event->{eventid});
          foreach my $img (glob($imgpath)) {
            $event->{image} .= sprintf("%s.png\n", basename($img, '.png'));
          }
        }

        $channel = $event->{channel};
        my $eventid = $obj->encodeEpgId($event->{eventid}, $channel);

        $event->{title} = gettext("No Title")
          unless($event->{title});
        $event->{description} = ""
          unless($event->{description});

        %{$dataHash->{$eventid}} = %{$event};

        $event = undef;
        $event->{channel} = $channel;
        next;
      } 
      elsif($line eq 'c') {
        # Finish this channel
        return ($dataHash,$channel,$channelname,$count+1)
          if(scalar keys %$dataHash);

        undef $event->{channel};
        undef $channel;
        undef $channelname;
      }

      my ($mark, $data) = $line =~ /^(\S)\s+(.+)/g;
      next unless($mark and $data);

      # Next channel 
      if($mark eq 'C') {
        if($channel) {
          debug 'Missing channel endtag c at line %d',$count;
          return ($dataHash,$channel,$channelname,$count) if(scalar keys %$dataHash);
        }
        undef $event->{channel};
        my $channel = (split(/\s+/, $data))[0];
        # import only known channels
        foreach my $ch (@{$channels}) {
          if($ch->[0] eq $channel) { 
            $event->{channel} = $channel;
            $channelname = $ch->[1];
            last;
          }
        }
      } elsif($mark eq 'E') {
        ($event->{eventid}, $event->{starttime}, $event->{duration}, $event->{tableid}, $event->{version}) = split(/\s+/, $data);
      } elsif($mark eq 'T') {
        $event->{title} = $data;
      } elsif($mark eq 'S') {
        $event->{subtitle} = $data;
      } elsif($mark eq 'D') {
        $event->{description} = $data;
        $event->{ldescription} = length($data);
      } elsif($mark eq 'X') {
        my @d = split(/\s+/, $data);
        if($d[0] eq '1') {
          $event->{video} .= $data;
        } else {
          $event->{audio} .= $data."\n";
        }
      } elsif($mark eq 'V') {
        $event->{vpstime} = $data;
      }
    }
#   debug 'Finish Read %d lines in EPG Database at %d', scalar @$vdata,$count;
    return ($dataHash,$channel,$channelname,$count);
}

# ------------------
sub search {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $data = shift;
    my $params = shift;

    my $tim = main::getModule('TIMERS');

    # Textsearch
    my $search;
    if($data) {
        if($params->{Where} eq 'title') {
            $search = buildsearch("e.title",$data);
        } elsif($params->{Where} eq 'titlesubtitle') {
            $search = buildsearch("e.title,e.subtitle",$data);
        } else {
            $search = buildsearch("e.title,e.subtitle,e.description",$data);
        }
    }

    # Channelsearch
    if($params->{channel}) {
        $search .= ' AND '
            if($search);
        $search .= sprintf('c.POS = %lu ', $params->{channel});
    }

    # Videoformat search
    if($params->{Videoformat} eq 'widescreen') {
        $search .= ' AND '
            if($search);
        $search .= 'e.video like "%%16:9%%" ';
    }

    # Audioformat search
    # XXX: Leider kann man an den Audioeintrag nicht richtig erkennnen
    # hab erst zu spt erkannt das diese Info aus dem tvm2vdr kommen ;(
#    if($params->{Audioformat} eq 'dts') {
#        $search .= ' AND '
#            if($search);
#        $search .= 'e.audio like "%%Digital%%" ';
#    }

    # MinLength search
    if($params->{MinLength}) {
        $search .= ' AND '
            if($search);
        $search .= sprintf('e.duration >= %d ', ($params->{MinLength}*60));
    }


    my $erg = [];
    if($search) {
        my $sql = qq|
    select
        e.eventid as Service,
        e.title as Title,
        e.subtitle as __Subtitle,
        c.Name as Channel,
        c.POS as __Pos,
        DATE_FORMAT(e.starttime, '%H:%i') as Start,
        DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(e.starttime) + e.duration), '%H:%i') as Stop,
        UNIX_TIMESTAMP(e.starttime) as Day,
        e.description,
        IF(e.vpstime!=0,DATE_FORMAT(e.vpstime, '%H:%i'),'') as __VPS
    from
        EPG as e,
        CHANNELS as c
    where
        e.channel_id = c.Id
        AND ( $search )
    order by
        starttime
        |;
    #dumper($sql);
        my $fields = fields($obj->{dbh}, $sql);

        $erg = $obj->{dbh}->selectall_arrayref($sql);
        unshift(@$erg, $fields);
    }
    return $console->table($erg, {timers => $tim->getEpgIds});
}

# ------------------
sub program {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $channel = shift || $obj->{dbh}->selectall_arrayref("select POS from CHANNELS limit 1")->[0][0];

    my $mod = main::getModule('CHANNELS');
    my $tim = main::getModule('TIMERS');

    my $cid;
    if($channel =~ /^\d+$/sig) {
        $cid = $mod->PosToChannel($channel)
            or return $console->message(sprintf(gettext("This channel '%s' does not exist in the database!"),$channel));
    } else {
        $cid = $mod->NameToChannel($channel)
            or return $console->message(sprintf(gettext("This channel '%s' does not exist in the database!"),$channel));
    }

    my $sql = qq|
select
    e.eventid as Service,
    e.title as Title,
    e.subtitle as __Subtitle,
    DATE_FORMAT(e.starttime, '%H:%i') as Start,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(e.starttime) + e.duration), '%H:%i') as Stop,
    UNIX_TIMESTAMP(e.starttime) as Day,
    e.description as __Description,
    e.video as __Video,
    e.audio as __Audio,
    IF(e.vpstime!=0,DATE_FORMAT(e.vpstime, '%H:%i'),'') as __VPS
from
    EPG as e, CHANNELS as c
where
    e.channel_id = c.Id and
    e.channel_id = '$cid'
order by
    starttime
|;
    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);

    $console->table($erg, {
                            channels => $mod->ChannelArray('Name'),
                            current => $mod->ChannelToPos($cid),
                            timers => $tim->getEpgIds
                           }
                    );
}

# ------------------
sub display {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $eventid = shift || return $console->message(gettext("No EventID to display the Event Programm! Please use display 'eventid'"));

    my %f = (
        'Id' => umlaute(gettext('Service')),
        'Title' => umlaute(gettext('Title')),
        'Subtitle' => umlaute(gettext('Subtitle')),
        'Channel' => umlaute(gettext('Channel')),
        'Start' => umlaute(gettext('Start')),
        'Stop' => umlaute(gettext('Stop')),
        'Description' => umlaute(gettext('Description')),
        'Percent' => umlaute(gettext('Percent')),
    );

    my $fields;
    my $erg;

    foreach my $table (qw/EPG OLDEPG/) {
    my $sql = qq|
select
    e.eventid as $f{'Id'},
    e.title as $f{'Title'},
    e.subtitle as $f{'Subtitle'},
    DATE_FORMAT(e.starttime, '%W: %H:%i %e.%c.%Y') as $f{'Start'},
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(e.starttime) + e.duration), '%W: %H:%i %e.%c.%Y') as $f{'Stop'},
    c.Name as $f{'Channel'},
    e.description as $f{'Description'},
    e.image as __Image,
    (unix_timestamp(e.starttime) + e.duration - unix_timestamp())/duration*100 as $f{'Percent'},
    e.video as __Video,
    e.audio as __Audio,
    IF(e.vpstime!=0,DATE_FORMAT(e.vpstime, '%H:%i'),'') as __VPS
from
    $table as e,CHANNELS as c
where
    e.channel_id = c.Id
    and eventid = '$eventid'
|;
    $fields = fields($obj->{dbh}, $sql);
    $erg = $obj->{dbh}->selectall_arrayref($sql);

    last
      if(scalar @{$erg} != 0 );
    }

    return $console->err(sprintf(gettext("No data for event '%d' present to display!"),$eventid))
      if(scalar @{$erg} == 0 );

    # Translate weekday for field start and stop
    my @wdays = (
          'Monday', gettext("Monday") ,
          'Tuesday', gettext("Tuesday") ,
          'Wednesday', gettext("Wednesday") ,
          'Thursday', gettext("Thursday") ,
          'Friday', gettext("Friday") ,
          'Saturday', gettext("Saturday") ,
          'Sunday', gettext("Sunday")
    );

    for (my $n = 0;$n < ((scalar @wdays)/2); ++$n ) {

        my $fday = $wdays[($n*2)];
        my $tday = $wdays[($n*2)+1];

        # halt here if'nt translation present
        last
            if($fday eq $tday);

        # Translate field start
        @$erg[0]->[3] =~ s/$fday/$tday/g;
        # Translate field stop
        @$erg[0]->[4] =~ s/$fday/$tday/g;
    }
    ######################
    unshift(@$erg, $fields);

    my $tim = main::getModule('TIMERS');
    $console->table($erg,{timers => $tim->getEpgIds});
}

# ------------------
sub runningNext {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $data   = shift;
    my $param   = shift || {};
    my $cgroups = main::getModule('CHANNELS')->ChannelGroupsArray('Name');
    my $cgrp    = $param->{cgrp} || $cgroups->[0][1]; # Erster GroupEintrag

    # Create temporary table
    $obj->{dbh}->do(qq|
CREATE TEMPORARY TABLE IF NOT EXISTS NEXTEPG (
    channel_id varchar(100) NOT NULL default '',
    nexttime datetime NOT NULL default '0000-00-00 00:00:00'
    )
|);
    # Remove old data
    $obj->{dbh}->do('delete from NEXTEPG');

    # Get channelid and starttime of next broadcasting
    $obj->{dbh}->do(qq|
INSERT INTO NEXTEPG select
    c.Id as channel_id,
    MIN(e.starttime) as nexttime
    FROM EPG as e, CHANNELS as c
    WHERE e.channel_id = c.Id
AND UNIX_TIMESTAMP(e.starttime) > UNIX_TIMESTAMP(NOW())
AND c.GRP = $cgrp

GROUP BY c.Id
|);

    my $sql =
qq|
select
    e.eventid as Service,
    e.title as Title,
    e.subtitle as __Subtitle,
    c.Name as Channel,
    c.POS as __POS,
    g.Name as __Channelgroup,
    DATE_FORMAT(e.starttime, "%H:%i") as Start,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(starttime) + e.duration), "%H:%i") as Stop,
    e.description as __Description,
    IF(e.vpstime!=0,DATE_FORMAT(e.vpstime, '%H:%i'),'') as __VPS
FROM
    EPG as e, CHANNELS as c, NEXTEPG as n, CHANNELGROUPS as g
WHERE
    e.channel_id = c.Id
    AND n.channel_id = c.Id
    AND c.GRP = g.Id
    AND e.starttime = n.nexttime
    AND c.GRP = $cgrp
ORDER BY
    c.POS|;
    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);

    my $tim = main::getModule('TIMERS');

    $console->table($erg,
        {
            timers => $tim->getEpgIds,
            periods => $obj->{periods},
            cgroups => $cgroups,
            channelgroup => $cgrp,
        }
    );
}

# ------------------
sub runningNow {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $zeit = shift || time;
    my $param   = shift || {};
    my $cgroups = main::getModule('CHANNELS')->ChannelGroupsArray('Name');
    my $cgrp    = $param->{cgrp} || $cgroups->[0][1]; # Erster GroupEintrag

    # i.e.: 635 --> 06:35
    $zeit = fmttime($zeit)
        if(length($zeit) <= 4);

    # i.e.: 06:35 --> timeinsecs
    if($zeit =~ /^\d+:\d+$/sig) {
        $zeit   = UnixDate(ParseDate($zeit),"%s") || time;
    }

    $zeit += 86400 if($zeit < time);
    $zeit++;

    my %f = (
        'Service' => umlaute(gettext('Service')),
        'Title' => umlaute(gettext('Title')),
        'Channel' => umlaute(gettext('Channel')),
        'Start' => umlaute(gettext('Start')),
        'Stop' => umlaute(gettext('Stop')),
        'Percent' => umlaute(gettext('Percent')),
    );
    my $sql =
qq|
select
    e.eventid as $f{'Service'},
    e.title as $f{'Title'},
    e.subtitle as __Subtitle,
    c.Name as $f{'Channel'},
    c.POS as __POS,
    g.Name as __Channelgroup,
    DATE_FORMAT(e.starttime, "%H:%i") as $f{'Start'},
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(starttime) + e.duration), "%H:%i") as $f{'Stop'},
    e.description as __Description,
    (unix_timestamp(e.starttime) + e.duration - unix_timestamp())/e.duration*100 as $f{'Percent'},
    IF(e.vpstime!=0,DATE_FORMAT(e.vpstime, '%H:%i'),'') as __VPS
FROM
    EPG as e, CHANNELS as c, CHANNELGROUPS as g
WHERE
    e.channel_id = c.Id
    AND c.GRP = g.Id
    AND $zeit BETWEEN UNIX_TIMESTAMP(e.starttime)
    AND (UNIX_TIMESTAMP(e.starttime) + e.duration)
    AND c.GRP = $cgrp
ORDER BY
    c.POS|;

    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);

    my $tim = main::getModule('TIMERS');
    $console->table($erg,
        {
            timers => $tim->getEpgIds,
            zeit => $zeit,
            periods => $obj->{periods},
            cgroups => $cgroups,
            channelgroup => $cgrp,
        }
    );
}

# ------------------
sub NowOnChannel {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $channel = shift || $obj->_actualChannel || return error('No Channel!');
    my $zeit = time;

    my $sql =
qq|
select
    e.eventid as Service,
    e.title as Title,
    e.subtitle as Subtitle,
    c.Name as Channel,
    c.POS as POS,
    e.video as __video,
    e.audio as __audio,
    DATE_FORMAT(e.starttime, "%a %d.%m") as StartDay,
    DATE_FORMAT(e.starttime, "%H:%i") as StartTime,
    (unix_timestamp(e.starttime) + e.duration - unix_timestamp())/e.duration*100 as __Percent,
    e.description as Description,
    IF(e.vpstime!=0,DATE_FORMAT(e.vpstime, '%H:%i'),'') as __VPS
FROM
    EPG as e, CHANNELS as c
WHERE
    e.channel_id = c.Id
    AND $zeit BETWEEN UNIX_TIMESTAMP(e.starttime)
    AND (UNIX_TIMESTAMP(e.starttime) + e.duration)
    AND c.POS = $channel
ORDER BY
    starttime
LIMIT 1
|;
#dumper($sql);
    my $erg = $obj->{dbh}->selectrow_hashref($sql);

    if(ref $console) {
        return $console->table($erg);
    } else {
        return $erg;
    }
}

# ------------------
sub _actualChannel {
# ------------------
    my $obj = shift  || return error ('No Object!' );

    my $erg = main::getModule('SVDRP')->command('chan');
    my ($chanpos, $channame) = $erg->[1] =~ /^250\s+(\d+)\s+(\S+)/sig;
    return $chanpos;
}

# ------------------
sub schema {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $zeit = shift || time;
    my $param   = shift || {};


    # i.e.: 635 --> 06:35
    $zeit = fmttime($zeit)
        if(length($zeit) <= 4);

    # i.e.: 06:35 --> timeinsecs
    if($zeit =~ /^\d+:\d+$/sig) {
        $zeit   = UnixDate(ParseDate($zeit),"%s") || time;
    }

    $zeit += 86400 if($zeit < time);
    $zeit++;
    my $zeitvon = $obj->toFullHour($zeit);

    my $zeitbis = $zeitvon + ($obj->{timeframe}*3600);
    my $cgroups = main::getModule('CHANNELS')->ChannelGroupsArray('Name');
    my $cgrp    = $param->{cgrp} || $cgroups->[0][1]; # Erster GroupEintrag

    my $sql =
qq|
select
    e.eventid as Service,
    e.title as Title,
    e.subtitle as __Subtitle,
    c.Name as Channel,
    c.POS as __POS,
    DATE_FORMAT(e.starttime, "%H:%i") as Start,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(starttime) + e.duration), "%H:%i") as Stop,
    (unix_timestamp(e.starttime) + e.duration - unix_timestamp())/e.duration*100 as Percent,
    e.description as __Description,
    UNIX_TIMESTAMP(starttime) as second_start,
    UNIX_TIMESTAMP(starttime) + e.duration as second_stop,
    e.video as __video,
    e.audio as __audio,
    e.image as __image
FROM
    EPG as e, CHANNELS as c
WHERE
    e.channel_id = c.Id
    AND
    (
        ( UNIX_TIMESTAMP(e.starttime) >= $zeitvon AND UNIX_TIMESTAMP(e.starttime) <= $zeitbis )
        OR
        ( UNIX_TIMESTAMP(e.starttime) + e.duration >= $zeitvon AND UNIX_TIMESTAMP(e.starttime) + e.duration <= $zeitbis )
        OR
        ( UNIX_TIMESTAMP(e.starttime) <= $zeitvon AND UNIX_TIMESTAMP(e.starttime) + e.duration >= $zeitbis )
    )
    AND
    c.GRP = $cgrp
ORDER BY
    c.POS,e.starttime
|;

    my $fields = fields($obj->{dbh}, $sql);
    my $erg = $obj->{dbh}->selectall_arrayref($sql);

    my $data = {};
    foreach my $c (@$erg) {
        push(@{$data->{$c->[4]}}, $c);
    }

    my $tim = main::getModule('TIMERS');
    $console->table($data,
        {
            timers => $tim->getEpgIds,
            runningTimer => $tim->getRunningTimer('NextEpgId'),
            zeitvon => $zeitvon,
            zeitbis => $zeitbis,
            periods => $obj->{periods},
            cgroups => $cgroups,
            channelgroup => $cgrp,
            HouresProSite => $obj->{timeframe}
        }
    );
}

# ------------------
sub checkOnTimer {
# ------------------
    my $obj = shift  || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $eid = shift  || return error('No Id');
    my $tim = main::getModule('TIMERS');

    my $sql = qq|
SELECT
    e.starttime as NextStartTime,
    ADDDATE(e.starttime, INTERVAL e.duration SECOND) as NextStopTime,
    LEFT(c.Source,1) as source,
    c.TID as transponderid
FROM
    EPG as e, CHANNELS as c
WHERE
    e.eventid = $eid
    and
    e.channel_id = c.Id
|;
    my $data = $obj->{dbh}->selectrow_hashref($sql)
        or return error('fehler:'.$obj->{dbh}->err);
    my $erg = $tim->checkOverlapping($data) || ['ok'];

    my $tmod = main::getModule('TIMERS');
    # Zeige den Title des Timers
    foreach (@$erg) { 
      $_ = $tmod->getTimerById((split(':', $_))[0])->{File}
        unless($_ eq 'ok'); 
    }

    $console->message(join(',',@$erg))
        if(ref $console);

}

# ------------------
sub toFullHour {
# ------------------
    my $obj = shift  || return error ('No Object!' );
    my $zeit = shift || return error ('No Time to convert!' );

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
                                             localtime($zeit);
    my $retzeit = timelocal(0, 0, $hour, $mday, $mon, $year);
    return $retzeit;
}


# ------------------
sub getId {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $id = shift || return error ('No Id!' );
    my $fields = shift || '*';

    # EPG
    my $sql = sprintf('select %s from EPG WHERE eventid = %lu', $fields, $id);
    my $erg = $obj->{dbh}->selectrow_hashref($sql);

    # OLDEPG
    unless($erg) {
        $sql = sprintf('select %s from OLDEPG WHERE eventid = %lu', $fields, $id);
        $erg = $obj->{dbh}->selectrow_hashref($sql);
    }

    unless($erg) {
        return debug("Event %d not exist!", $id);
    } else {
        return $erg;
    }
}

1;
