package XXV::MODULES::TIMERS;

use strict;
use Tools;
use POSIX ":sys_wait_h", qw(strftime mktime);
use Locale::gettext;

# This module method must exist for XXV
# ------------------
sub module {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $args = {
        Name => 'TIMERS',
        Prereq => {
            # 'Perl::Module' => 'Description',
        },
        Description => gettext('This module parse the timers.conf and save this in the database.'),
        Version => '0.81',
        Date => '23.08.2006',
        Author => 'xpix',
        Status => sub{ $obj->status(@_) },
        Preferences => {
            interval => {
                description => gettext('The interval to parse timers to database in seconds'),
                default     => 300,
                type        => 'integer',
                required    => gettext("This is required!"),
            },
            prevminutes => {
                description => gettext('Buffer time in minutes before the scheduled end of the recorded program.'),
                default     => 5,
                type        => 'integer',
            },
            afterminutes => {
                description => gettext('Buffer time in minutes after the scheduled end of the recorded program.'),
                default     => 5,
                type        => 'integer',
            },
            Priority => {
                description => gettext('Defining the priority of this timer and of recordings created by this timer.'),
                default     => 50,
                type        => 'integer',
            },
            Lifetime => {
                description => gettext('The guaranteed lifetime (in days) of a recording created by this timer'),
                default     => 50,
                type        => 'integer',
            },
            DVBCards => {
                description => gettext('How much DVB cards in your system?'),
                default     => 1,
                type        => 'integer',
            },
            deactive => {
                description => gettext('Delete deactiv timer after his endtime?'),
                default     => 'n',
                type        => 'confirm',
            },
            usevpstime => {
                description => gettext('Use VPS start time?'),
                default     => 'n',
                type        => 'confirm',
            },
        },
        Commands => {
            tlist => {
                description => gettext("List timers 'tid'"),
                short       => 'tl',
                callback    => sub{ $obj->list(@_) },
                DenyClass   => 'tlist',
            },
            tsearch => {
                description => gettext("Search timers 'text'"),
                short       => 'ts',
                callback    => sub{ $obj->list(@_) },
                DenyClass   => 'tlist',
            },
            tupdate => {
                description => gettext("Read timers and write into database"),
                short       => 'tu',
                callback    => sub{ $obj->readData(@_)},
                Level       => 'user',
                DenyClass   => 'tedit',
            },
            tnew => {
                description => gettext("Create timer 'eid'"),
                short       => 'tn',
                callback    => sub{ $obj->newTimer(@_) },
                Level       => 'user',
                DenyClass   => 'tedit',
            },
            tedit => {
                description => gettext("Edit timer 'tid'"),
                short       => 'te',
                callback    => sub{ $obj->editTimer(@_) },
                Level       => 'user',
                DenyClass   => 'tedit',
            },
            tdelete => {
                description => gettext("Delete timer 'tid'"),
                short       => 'td',
                callback    => sub{ $obj->deleteTimer(@_) },
                Level       => 'user',
                DenyClass   => 'tedit',
            },
            ttoggle => {
                description => gettext("Activate/Deactive timer 'tid'"),
                short       => 'tt',
                callback    => sub{ $obj->toggleTimer(@_) },
                Level       => 'user',
                DenyClass   => 'tedit',
            },
        },
        RegEvent    => {
            'newTimerfromUser' => {
                Descr => gettext('Create event entries, if a new timer created by user.'),

                # You have this choices (harmless is default):
                # 'harmless', 'intresting', 'veryintresting', 'important', 'veryimportant'
                Level => 'intresting',

                # Search for a spezial Event.
                # I.e.: Search for an LogEvent with match
                # "Sub=>text" = subroutine =~ /text/
                # "Msg=>text" = logmessage =~ /text/
                # "Mod=>text" = modname =~ /text/
                SearchForEvent => {
                    Msg => 'User Timer',
                },
                # Search for a Match and extract the information
                # of the TimerId
                # ...
                Match => {
                    TimerId => qr/TimerId\:\s+\"(\d+)\"/s,
                },
                Actions => [
                    q|sub{  my $args = shift;
                            my $event = shift;
                            my $timer  = getDataById($args->{TimerId}, 'TIMERS', 'Id');
                            return if($timer->{AutotimerId});
                            my $title = sprintf(gettext("New timer found: %s"),$timer->{File});
                            my $description = sprintf(gettext("Description: %s\nAt: %s to %s"),
                                $timer->{Summary},
                                $timer->{NextStartTime},
                                fmttime($timer->{Stop})
                                );

                            main::getModule('REPORT')->news($title, $description, "display", $timer->{NextEpgId}, $event->{Level});
                        }
                    |,
                    q|sub{  my $args = shift;
                            my $event = shift;
                            my $timer  = getDataById($args->{TimerId}, 'TIMERS', 'Id');
                            my $soap = main::getModule('SHARE');
                            my $level = 1;
                            if($timer->{AutotimerId}) {
                                $level = ($timer->{Prio} < 50 ? 2 : 3);
                            } else {
                                $level = ($timer->{Prio} < 50 ? 4 : 5);
                            }

                            if($timer->{NextEpgId}) {
                                my $event = main::getModule('EPG')->getId($timer->{NextEpgId}, 'UNIX_TIMESTAMP(starttime) + duration as STOPTIME');
                                $soap->setEventLevel($timer->{NextEpgId}, $level, $event->{STOPTIME});
                            }
                        }|,
                ],

            },
            'deleteTimer' => {
                Descr => gettext('Create event entries, if timer deleted by user.'),
                Level => 'intresting',
                SearchForEvent => {
                    Msg => 'delt',
                },
                Match => {
                    TimerId => qr/delt\s+(\d+)/s,
                },
                Actions => [
                    q|sub{  my $args = shift;
                            my $event = shift;
                            my $timer  = getDataById($args->{TimerId}, 'TIMERS', 'Id');
                            my $title = sprintf(gettext("Timer deleted: %s"),$timer->{File});
                            my $description = sprintf(gettext("Description: %s\nAt: %s to %s"),
                                $timer->{Summary},
                                $timer->{NextStartTime},
                                fmttime($timer->{Stop})
                                );

                            main::getModule('REPORT')->news($title, $description, "display", $timer->{NextEpgId}, $event->{Level});
                        }
                    |,
                    q|sub{  my $args = shift;
                            my $event = shift;
                            my $timer  = getDataById($args->{TimerId}, 'TIMERS', 'Id');
                            my $soap = main::getModule('SHARE');
                            my $level = 1;

                            if($timer->{NextEpgId}) {
                                my $event = main::getModule('EPG')->getId($timer->{NextEpgId}, 'UNIX_TIMESTAMP(starttime) + duration as STOPTIME');
                                $soap->setEventLevel($timer->{NextEpgId}, $level, $event->{STOPTIME});
                            }
                        }|,
                ],
            },
            'toggleTimer' => {
                Descr => gettext('Create event entries, if timer toggled by user.'),
                Level => 'intresting',
                SearchForEvent => {
                    Msg => 'modt',
                },
                Match => {
                    TimerId => qr/modt\s+(\d+)\s(on|off)/s,
                    Type    => qr/modt\s+\d+\s+(on|off)/s,
                },
                Actions => [
                    q|sub{  my $args = shift;
                            my $event = shift;
                            my $timer  = getDataById($args->{TimerId}, 'TIMERS', 'Id');
                            my $title = sprintf(gettext("Timer toggled: %s to %s"),$timer->{File},$args->{Type});
                            my $description = sprintf(gettext("Description: %s\nAt: %s to %s"),
                                $timer->{Summary},
                                $timer->{NextStartTime},
                                fmttime($timer->{Stop})
                                );

                            main::getModule('REPORT')->news($title, $description, "display", $timer->{NextEpgId}, $event->{Level});
                        }
                    |,
                    q|sub{  my $args = shift;
                            my $event = shift;
                            my $timer  = getDataById($args->{TimerId}, 'TIMERS', 'Id');
                            my $soap = main::getModule('SHARE');
                            my $level = ($args->{Type} eq 'off' ? 1 : 2);
                            if($timer->{AutotimerId} and $args->{Type} eq 'on') {
                                $level = ($timer->{Prio} < 50 ? 2 : 3);
                            } elsif($args->{Type} eq 'on') {
                                $level = ($timer->{Prio} < 50 ? 4 : 5);
                            }

                            if($timer->{NextEpgId}) {
                                my $event = main::getModule('EPG')->getId($timer->{NextEpgId}, 'UNIX_TIMESTAMP(starttime) + duration as STOPTIME');
                                $soap->setEventLevel($timer->{NextEpgId}, $level, $event->{STOPTIME});
                            }
                        }|,
                ],
            },
            'updateTimer' => {
                Descr => gettext('Create event entries, if timer updated.'),
                Level => 'harmless',
                SearchForEvent => {
                    Msg => 'Reread',
                },
                Match => {
                    HighId => qr/Reread\s+(\d+)\s+timers/s,
                },
                Actions => [
                    q|sub{  my $args = shift;
                            my $event = shift;
                            my $soap = main::getModule('SHARE');
                            my $epg = main::getModule('EPG');
                            for (my $i = 1; $i<=$args->{HighId}; $i++) {
                                my $timer  = getDataById($i, 'TIMERS', 'Id');

                                my $level = ($timer->{Status} ? 1 : 2);
                                if($timer->{AutotimerId} and $timer->{Status}) {
                                    $level = ($timer->{Prio} < 50 ? 2 : 3);
                                } elsif($timer->{Status}) {
                                    $level = ($timer->{Prio} < 50 ? 4 : 5);
                                }

                                if($timer->{NextEpgId}) {
                                    my $event = $epg->getId($timer->{NextEpgId}, 'UNIX_TIMESTAMP(starttime) + duration as STOPTIME');
                                    $soap->setEventLevel($timer->{NextEpgId}, $level, $event->{STOPTIME});
                                }
                            }
                    }|,
                ],
            },
        },
    };
    return $args;
}

# ------------------
sub status {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;
    my $lastReportTime = shift || 0;

    my $sql = "select count(*) from TIMERS";
    my $erg = $obj->{dbh}->selectall_arrayref($sql);

    return {
        message => sprintf(gettext('%d Timers exists.'), $erg->[0][0]),
    };
}

# ------------------
sub new {
# ------------------
	my($class, %attr) = @_;
	my $self = {};
	bless($self, $class);

    # paths
    $self->{paths} = delete $attr{'-paths'};

	# who am I
    $self->{MOD} = $self->module;

    # all configvalues to $self without parents (important for ConfigModule)
    map {
        $self->{$_} = $attr{'-config'}->{$self->{MOD}->{Name}}->{$_};
        $self->{$_} = $self->{MOD}->{Preferences}->{$_}->{default} unless($self->{$_});
    } keys %{$self->{MOD}->{Preferences}};

    # Try to use the Requirments
    map {
        eval "use $_";
        return panic("\nCan not load Module: $_\nPlease install this module on your System:\nperl -MCPAN -e 'install $_'") if($@);
    } keys %{$self->{MOD}->{Prereq}};

    # read the DB Handle
    $self->{dbh} = delete $attr{'-dbh'};

    # The Initprocess
    my $erg = $self->_init or return error('Problem to initialze Module');

    # Interval to read timers and put to DB
    Event->timer(
        interval => $self->{interval},
        prio => 6,  # -1 very hard ... 6 very low
        cb => sub{
            $self->readData();
        },
    );

	return $self;
}

# ------------------
sub _init {
# ------------------
    my $obj = shift || return error ('No Object!' );

    return 0, panic("Session to database is'nt connected")
      unless($obj->{dbh});

    # Old Version
    if(tableExists($obj->{dbh}, 'TIMERS')) {
        my $fields = fields($obj->{dbh}, 'select * from TIMERS');
        $obj->{dbh}->do('drop table TIMERS')
            unless(scalar @$fields == 17);
    }

    # Look for table or create this table
    my $version = main::getVersion;
    $obj->{dbh}->do(qq|
      CREATE TABLE IF NOT EXISTS TIMERS (
          Id int(11) unsigned NOT NULL,
          Status char(1) default 1,
          ChannelID varchar(100) NOT NULL default '',
          Day varchar(20) default '-------',
          Start int(11) unsigned,
          Stop int(11) unsigned,
          Priority tinyint(2),
          Lifetime tinyint(2),
          File text,
          Summary text default '',
          NextStartTime datetime,
          NextStopTime datetime,
          Collision varchar(100) default '0',
          NextEpgId bigint unsigned default '0',
          AutotimerId int(11) unsigned default '0',
          addtime timestamp,
          Checked char(1) default 0,
          PRIMARY KEY  (Id)
        ) COMMENT = '$version'
    |);

    $obj->{LastRefreshTime} = time;
    $obj->{newTimerFormat} = 0;

# NUR ZUM TESTEN !!!!
# use NetServer::Portal; 'NetServer::Portal'->default_start();
# debug("NetServer::Portal listening on port %d", (7000+($$ % 1000)));

    main::after(sub{
        $obj->{svdrp} = main::getModule('SVDRP');
        $obj->readData();
    }, "TIMERS: Store timers in database ...", 50);


    return 1;
}

# ------------------
sub saveTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $data = shift || return error('No Data to Save!');
    my $timerid = shift || 0;

    my $status = ($data->{Activ} eq 'y' ? 1 : 0);
       $status |= ($data->{VPS} eq 'y' ? 4 : 0);

    $data->{File} =~ s/:/|/g;
    $data->{File} =~ s/(\r|\n)//sig;

    my $erg = $obj->{svdrp}->command(
        sprintf("%s %s:%s:%s:%s:%s:%s:%s:%s:%s",
            $timerid ? "modt $timerid" : "newt",
            $status,
            $data->{ChannelID},
            $data->{Day},
            $data->{Start},
            $data->{Stop},
            int($data->{Priority}),
            int($data->{Lifetime}),
            $data->{File},
            ($data->{Summary} || '')
        )
    );

    # Save shortly this timer in DB if this only a new timer (at)
    # Very Important for Autotimer!
    my $pos = $1 if($erg->[1] =~ /^250\s+(\d+)/);
    if(! $timerid and $pos) {
        $obj->insert([
                $status,
                $data->{ChannelID},
                $data->{Day},
                $data->{Start},
                $data->{Stop},
                int($data->{Priority}),
                int($data->{Lifetime}),
                $data->{File},
	            ($data->{Summary} || '')
                ], $pos);
    }

    event('Save timer "%s" with TimerId: "%d"', $data->{File}, $pos);

    return $erg;
}

# ------------------
sub newTimer {
# ------------------
    my $obj     = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $epgid   = shift || 0;
    my $epg     = shift || 0;


    if($epgid and not ref $epg) {
        $epg = $obj->{dbh}->selectrow_hashref(qq|
SELECT
    eventid,
    channel_id,
    description as Summary,
    CONCAT_WS('~', title, subtitle) as File,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(starttime) - ( $obj->{prevminutes} * 60 )), '%d') as Day,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(starttime) - ( $obj->{prevminutes} * 60 )), '%H%i') as Start,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(starttime) + duration + ( $obj->{afterminutes} * 60 )), '%H%i') as Stop,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(vpstime)), '%H%i') as VpsStart,
    DATE_FORMAT(FROM_UNIXTIME(UNIX_TIMESTAMP(vpstime) + duration), '%H%i') as VpsStop
FROM
    EPG
WHERE
    eventid = $epgid
|
        ) or return $console->message(sprintf(gettext("Event ID '%s' does not exist in the database!"),$epgid));
    }
    if(not ref $epg) {
		my $t = time;
   	    $epg = {
            channel_id => '',
            File => '',
            Summary => '',
            Day => $obj->{newTimerFormat}?my_strftime("%Y-%m-%d",$t):my_strftime("%d",$t),
            Start => my_strftime("%H%M",$t),
            Stop => my_strftime("%H%M",$t)
    	};
    }

    $epg->{Status} = '1'
         if(not defined $epg->{Status});
    $epg->{Priority} = $obj->{Priority}
         if(not defined $epg->{Priority});
    $epg->{Lifetime} = $obj->{Lifetime}
         if(not defined $epg->{Lifetime});
	if(main::getVdrVersion() >= 10344) {
    	$epg->{desc} = $epg->{Summary};
    	$epg->{Summary} = ""
    }
    if($epg->{VpsStart} && $obj->{usevpstime} eq 'y') {
        $epg->{Status} |= 4;
        $epg->{Start} = $epg->{VpsStart};
        $epg->{Stop} = $epg->{VpsStop};
    }

    $obj->editTimer($watcher, $console, 0, $epg);
}

# ------------------
sub editTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $timerid = shift || 0;   # If timerid the edittimer
    my $data    = shift || 0;  # Data for defaults

    my $epg;
    if($timerid and not ref $data) {
        $epg = $obj->{dbh}->selectrow_hashref("select Id, ChannelID as channel_id, File, Summary, Start, Stop, Day, Priority, Lifetime, Status from TIMERS where Id = $timerid")
            or return $console->message(sprintf(gettext("Timer ID '%s' does not exist in the database!"),$timerid));
    } elsif (ref $data eq 'HASH') {
        $epg = $data;
    }

    $epg->{Summary} =~ s/(\r|\n)//sig
        if(defined $epg->{Summary});

    my $mod = main::getModule('CHANNELS');

    my $questions = [
        'Id' => {
            typ     => 'hidden',
            def     => $epg->{Id} || 0,
        },
        'Activ' => {
            typ     => 'confirm',
            def     => (defined $epg->{Status} and ($epg->{Status} & 1) ? 'y' : 'n'),
            msg     => gettext('Switch this timer on?'),
        },
        'VPS' => {
            typ     => 'confirm',
            def     => (defined $epg->{Status} and ($epg->{Status} & 4) ? 'y' : 'n'),
            msg     => gettext('VPS for this timer on?'),
        },
        'ChannelID' => {
            typ     => 'list',
            def     => $mod->ChannelToPos($epg->{channel_id}),
            choices => $mod->ChannelArray('Name'),
            msg     => gettext('Which channel should recorded?'),
            req     => gettext("This is required!"),
            check   => sub{
                my $value = shift || return;

                if(my $ch = $mod->PosToChannel($value) || $mod->NameToChannel($value) ) {
                    $epg->{Channel} = $value;
                    return $ch;
                } elsif( ! $mod->NameToChannel($value)) {
                    return undef, sprintf(gettext("This channel '%s' does not exist!"),$value);
                } else {
                   return undef, gettext("This is required!");
                }
            },
        },
        'Day' => {
            typ     => 'date',
            def     => $epg->{Day},
            msg     => gettext("Please enter a day (1 to 31) or the weekday in format 'MDMDFSS'."),
            req     => gettext("This is required!"),
            check   => sub{
                my $value = shift || return;
                if(($value =~ /^\d+$/ and int($value) <= 31 and int($value) > 0)            # 13
                        or ($obj->{newTimerFormat} and $value =~ /^\d{4}\-\d{2}-\d{2}$/sig) # 2005-03-13
                        or $value =~ /^\S{7}\@\d{4}\-\d{2}-\d{2}$/sig                       # MTWTFSS@2005-03-13
                        or $value =~ /^\S{7}\@\d{2}$/sig                                    # MTWTFSS@13
                        or $value =~ /^\S{7}$/) {                                           # MTWTFSS
                    return $value;
                } else {
                    return undef, gettext('No right day or corrupt format!');
                }
            },
        },
        'Start' => {
            typ     => 'string',
            def     => sub{
                return fmttime($epg->{Start});
                },
            msg     => gettext("Starttime in format 'HH:MM'"),
            check   => sub{
                my $value = shift;
                $value = fmttime($value) if($value =~ /^\d+$/sig);
                return undef, gettext('No right time!') if($value !~ /^\d+:\d+$/sig);
                my @v = split(':', $value);
                $value = sprintf('%02d%02d',$v[0],$v[1]);
                if(int($value) < 2400 and int($value) >= 0) {
                    return sprintf('%04d',$value);
                } else {
                    return undef, gettext('No right time!');
                }
            },
        },
        'Stop' => {
            typ     => 'string',
            def     => sub{
                    return fmttime($epg->{Stop} || 0 );
                },
            msg     => gettext("Endtime in format 'HH:MM'"),
            check   => sub{
                my $value = shift;
                $value = fmttime($value) if($value =~ /^\d+$/sig);
                return undef, gettext('No right time!') if($value !~ /^\d+:\d+$/sig);
                my @v = split(':', $value);
                $value = sprintf('%02d%02d',$v[0],$v[1]);
                if(int($value) < 2400 and int($value) >= 0) {
                    return sprintf('%04d',$value);
                } else {
                    return undef, gettext('No right time!');
                }
            },
        },
        'Priority' => {
            typ     => 'integer',
            msg     => sprintf(gettext('Priority (0 .. %d)'),$console->{USER}->{MaxPriority} ? $console->{USER}->{MaxPriority} : 99 ),
            def     => int($epg->{Priority}),
            check   => sub{
                my $value = shift || 0;
                if($value =~ /^\d+$/sig and $value >= 0 and $value < 100) {
                    if($console->{USER}->{MaxPriority} and $value > $console->{USER}->{MaxPriority}) {
                        return undef, sprintf(gettext('Sorry, but maximum priority is limited on %d!'), $console->{USER}->{MaxPriority});
                    }
                    return int($value);
                } else {
                    return undef, gettext('No right Value!');
                }
            },
        },
        'Lifetime' => {
            typ     => 'integer',
            msg     => sprintf(gettext('Lifetime (0 .. %d)'),$console->{USER}->{MaxLifeTime} ? $console->{USER}->{MaxLifeTime} : 99 ),
            def     => int($epg->{Lifetime}),
            check   => sub{
                my $value = shift || 0;
                if($value =~ /^\d+$/sig and $value >= 0 and $value < 100) {
                    if($console->{USER}->{MaxLifeTime} and $value > $console->{USER}->{MaxLifeTime}) {
                        return undef, sprintf(gettext('Sorry, but maximum lifetime is limited on %d!'), $console->{USER}->{MaxLifeTime});
                    }
                    return int($value);
                } else {
                    return undef, gettext('No right Value!');
                }
            },
        },
        'File' => {
            msg     => gettext('Title of recording'),
            def     => $epg->{File},
            req     => gettext("This is required!"),
        },
    ];

    my $Summary = $epg->{Summary};
    $Summary =~s/\#~AT\[(\d+)\]//g;

	if(main::getVdrVersion() >= 10344){
        if($epg->{Id} || $epg->{desc}) {
            my $desc = $epg->{desc} || $obj->getEpgDesc($epg->{Id});
            if($desc) {
        		push(@$questions,
        		'Description' => {
                    msg   =>  gettext('Description'),
                    typ     => 'string',
                    def   => $desc,
                    readonly => 1
                });
            }
        }

		push(@$questions,
		'Summary' => {
            typ     => 'hidden',
            def   => $Summary,
        });
	} else {
		push(@$questions,
		'Summary' => {
            msg   =>  gettext('Additional description'),
            def   => $Summary,
            check   => sub{
                my $value = shift || return;
                $value =~ s/(\r|\n)//sig;
                return $value;
            },
        });
	}
    # Ask Questions
    my $datasave = $console->question(($timerid ? gettext('Edit timer')
                                                : gettext('New timer')), $questions, $data);

    if(ref $datasave eq 'HASH') {
        my $erg = $obj->saveTimer($datasave, $timerid);

        my $error;
        foreach my $zeile (@$erg) {
            if($zeile =~ /^(\d{3})\s+(.+)/) {
                $error = $2 if(int($1) >= 500);
            }
        }

        unless($error) {
            lg sprintf('%s timer with title "%s" is saved from User: %s',
                ($timerid ? 'Changed' : 'New'),
                $data->{File},
                $console->{USER}->{Name}
                );
                $console->message($erg);
        } else {
            error sprintf('%s timer with title "%s" does\'nt saved : %s',
                ($timerid ? 'Changed' : 'New'),
                $data->{File},
                $error
                );
                $console->err($erg);
        }
        $obj->readData();
        $console->redirect({url => $console->{browser}->{Referer}, wait => 2})
            if($console->typ eq 'HTML');
    }
}

# ------------------
sub deleteTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;
    my $timerid = shift || return $console->err(gettext("No Timer ID to delete! Please use tdelete 'tid'"));   # If timerid the edittimer
    my $answer  = shift || 0;

    my @timers  = reverse sort{ $a <=> $b } split(/[^0-9]/, $timerid);

    my $sql = sprintf("select * from TIMERS where Id in (%s)", join(',', @timers));
    my $data = $obj->{dbh}->selectall_hashref($sql, 'Id');

    my $mod = main::getModule('CHANNELS') or return;
    foreach my $tid (@timers) {
        unless(exists $data->{$tid}) {
            $console->err(sprintf(gettext("Timer with number '%s' does not exist in the database!"), $tid));
            next;
        }

        $data->{$tid}->{Channel} = $mod->ChannelToName($data->{ChannelID});

        if(ref $console and $console->{TYP} eq 'CONSOLE') {
            $console->table($data->{$tid});
            my $confirm = $console->confirm({
                typ   => 'confirm',
                def   => 'y',
                msg   => gettext('Are you sure to delete this timer?'),
            }, $answer);
            next if(! $answer eq 'y');
        }

        lg sprintf('Delete timer with title "%s" from User: %s',
            $data->{$tid}->{File},
            $console->{USER}->{Name}
            );

        $obj->{svdrp}->queue_cmds("delt $tid"); # Sammeln der Kommandos
    }

    if($obj->{svdrp}->queue_cmds('COUNT')) {
        my $erg = $obj->{svdrp}->queue_cmds("CALL"); # Aufrufen der Kommandos
        $console->msg($erg, $obj->{svdrp}->err)
            if(ref $console);

        sleep(1);

        $obj->readData();

        $console->redirect({url => $console->{browser}->{Referer}, wait => 1})
            if(ref $console and $console->typ eq 'HTML');
    } else {
        $console->err(gettext("No timer to delete!"));
    }

    return 1;
}

# ------------------
sub toggleTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $timerid = shift || return $console->err(gettext("No Timer ID to toggle! Please use ttoggle 'id'"));   # If timerid the edittimer

    my @timers  = reverse sort{ $a <=> $b } split(/[^0-9]/, $timerid);

    my $sql = sprintf("select * from TIMERS where Id in (%s)", join(',', @timers));
    my $data = $obj->{dbh}->selectall_hashref($sql, 'Id');

    for my $timer (@timers) {

        unless(exists $data->{$timer}) {
            $console->err(sprintf(gettext("Timer with number '%s' does not exist in the database!"), $timer));
            next;
        }

    	my $status = (($data->{$timer}->{Status} & 1) ? 'off' : 'on');

        lg sprintf('Timer with title "%s" is %s from User: %s',
            $data->{$timer}->{File},
            ($status eq 'on' ? 'activated' : 'deactivated'),
            $data->{File},
            $console->{USER}->{Name}
            );

        $obj->{svdrp}->queue_cmds("modt $data->{$timer}->{Id} $status"); # Sammeln der Kommandos
    }

    if($obj->{svdrp}->queue_cmds('COUNT')) {

        my $erg = $obj->{svdrp}->queue_cmds("CALL"); # Aufrufen der Kommandos
        $console->msg($erg, $obj->{svdrp}->err)
            if(ref $console);
        $obj->readData($watcher, $console);
        $console->redirect({url => $console->{browser}->{Referer}, wait => 1})
            if(ref $console and $console->typ eq 'HTML');
        return 1;
    } else {
        $console->err(gettext('No timer to toggle!'));
        return undef;
    }
}


# ------------------
sub insert {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $data = shift || return;
    my $pos = shift || return;
    my $checked = shift || 0;

    # import only status which used from vdr and thereby exclude eventid from vdradmin
    $data->[0] &= 15;

    # change pos to channelid, because change to telnet reader
    $data->[1] = $obj->{channels}->{$data->[1]}->{Id}
        if(index($data->[1], '-') < 0);

    # POS
    unshift(@$data, $pos);
    $data->[8] =~ s/\|/\:/g;

    # NextTime
    my $nexttime = $obj->getNextTime( $data->[3], $data->[4], $data->[5] )
        or return error(sprintf("Can not get the nextime for this data: %s", join(' ', @$data)));
    push(@$data, $nexttime->{start}, $nexttime->{stop});

    # AutotimerId
    my $atxt = (split('~', $data->[9]))[-1];
    my $aid = $1 if(defined $atxt and $atxt =~ /AT\[(\d+)\]/);
    push(@$data, $aid || 0);

    # checked
    push(@$data, $checked);

    my $sth = $obj->{dbh}->prepare('REPLACE INTO TIMERS VALUES (?,?,?,?,?,?,?,?,?,?,FROM_UNIXTIME(?), FROM_UNIXTIME(?),0,0,?,NOW(),?)');
    $sth->execute( @$data );
}


# Read from svdrp (better for future development)
# ------------------
sub readData {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift;
    my $console = shift;

    # Search for old and deactivated Timers and delete this
    $obj->getOldDeactivTimer()
        if($obj->{deactive} eq 'y');

    # Search for correct times
    $obj->getCheckTimer();

    my $oldTimers = &getDataByTable('TIMERS');

    $obj->{dbh}->do('DELETE FROM TIMERS');

    # read from svdrp, because the
    # vdr edit the timers.conf to lazy ;)
    $obj->{channels} = main::getModule('CHANNELS')->ChannelHash('POS');
    my $tlist = $obj->{svdrp}->command('lstt');

    my $c = 0;
    foreach my $line (@$tlist) {
        next if(! $line or $line =~ /^22/);
        $line =~ s/^\d+[- ]+\d+\s//sig;
        $c++;
        my @data = split(':', $line, 9);

        $obj->insert(\@data, $c, 1)
            if(scalar @data > 2);
    }

    # Search for overlapping Timers
    my $overlapping = $obj->getOverlappingTimer();

    # Search for next EpgId
    $obj->getNextEpgId($oldTimers);

    # Get timers by Autotimer
    my $aids = getDataByFields('AUTOTIMER', 'Id');
    $obj->getTimersByAutotimer($aids);

    # Get new timers by User
    if($oldTimers) {
        my $timers = $obj->getNewTimers($oldTimers);
        foreach my $timerdata (@$timers) {
            event('New User Timer "%s" with TimerId: "%d"', $timerdata->{File}, $timerdata->{Id});
        }
    }

    $obj->{REGISTER}++;
    if(scalar keys %$oldTimers != $c or $obj->{REGISTER} == 2) {
        # Event to signal we are finish to read
        event(sprintf('Reread %d timers and written to DB!', $c));
    }

    $console->message(sprintf(gettext("Write %d timers in database."), $c), {overlapping => $overlapping})
        if(ref $console);

    $console->redirect({url => $console->{browser}->{Referer}, wait => 1})
        if(ref $console and $console->typ eq 'HTML');

    $obj->{LastRefreshTime} = time;

    return 1;
}


# ------------------
sub list {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $watcher = shift || return error ('No Watcher!');
    my $console = shift || return error ('No Console');
    my $text    = shift || '';

		my $in = '';
		if($text and $text =~ /[0-9]+/) {
	    $in = "and t.Id in ($text)";
		} elsif($text and $text =~ /[a-z]+/) {
	    $in = sprintf('and ( %s )', buildsearch("t.File,t.Summary",$text));
		}
    my %f = (
        'Id' => umlaute(gettext('Sv')),
        'Status' => umlaute(gettext('Status')),
        'Prg' => umlaute(gettext('Prg')),
        'Channel' => umlaute(gettext('Channel')),
        'Start' => umlaute(gettext('Start')),
        'Stop' => umlaute(gettext('Stop')),
        'File' => umlaute(gettext('File')),
        'Priority' => umlaute(gettext('Priority')),
    );

    my $sql = qq|
SELECT
    t.Id as $f{'Id'},
    t.Status as $f{'Status'},
    c.Name as $f{'Channel'},
    c.Pos as __Pos,
    t.Day as $f{'Prg'},
    t.Start as $f{'Start'},
    t.Stop as $f{'Stop'},
    t.File as $f{'File'},
    t.Priority as $f{'Priority'},
    UNIX_TIMESTAMP(t.NextStartTime) as __Day,
    t.Collision as __Collision,
    t.NextEpgId as __NextEpgId,
    t.AutotimerId as __AutotimerId,
	UNIX_TIMESTAMP(t.NextStopTime) - UNIX_TIMESTAMP(t.NextStartTime) as __Duration
FROM
    TIMERS as t,
    CHANNELS as c
WHERE
    t.ChannelID = c.Id
    $in
ORDER BY
    t.NextStartTime|;

    my $fields = fields($obj->{dbh}, $sql);

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    unshift(@$erg, $fields);
    $console->table($erg, {
        runningTimer => $obj->getRunningTimer,
        cards => $obj->{DVBCards},
		    capacity => main::getModule('RECORDS')->{CapacityFree},
    });
}

# ------------------
sub getTimerById {
# ------------------
    my $obj = shift  || return error ('No Object!' );
    my $tid = shift  || return error ('No TimerId!' );

    my $sql = qq|
SELECT
    t.Id,
    t.Status,
    c.Name as Channel,
    c.Pos as __Pos,
    t.Day as Prg,
    t.Start,
    t.Stop,
    t.File,
    t.Priority,
    UNIX_TIMESTAMP(t.NextStartTime) as Day,
    t.Collision as Collision,
    t.NextEpgId as NextEpgId,
    t.AutotimerId as AutotimerId
FROM
    TIMERS as t,
    CHANNELS as c
WHERE
    t.ChannelID = c.Id
    and t.Id = $tid
|;

    my $erg = $obj->{dbh}->selectrow_hashref($sql);
    return $erg;
}


# ------------------
sub getRunningTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
		my $rowname = shift || 'Id';
    my $sql = "select * from TIMERS where NOW() between NextStartTime and NextStopTime AND (Status & 1) ";
    my $erg = $obj->{dbh}->selectall_hashref($sql, $rowname);
    return $erg;
}

# ------------------
sub getNewTimers {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $oldTimers = shift || return;

    my $ret = [];
    foreach my $timerid (keys %$oldTimers) {
        if(! $oldTimers->{$timerid}->{Checked}) {
            push(@$ret, $oldTimers->{$timerid});
        }
    }
    return $ret;
}

# ------------------
sub getOldDeactivTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $sql = "select Id from TIMERS where not (Status & 1) and UNIX_TIMESTAMP(NextStopTime) > UNIX_TIMESTAMP() + (60*60*24*28)";
    my $erg = $obj->{dbh}->selectall_hashref($sql, 'Id');

    foreach my $t (reverse sort {$a <=> $b} keys %$erg) {
        $obj->{svdrp}->queue_cmds("delt $t");
    }
    $obj->{svdrp}->queue_cmds("CALL")
        if($obj->{svdrp}->queue_cmds("COUNT"));
    return $erg;
}

# ------------------
sub getCheckTimer {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $sql = "select * from TIMERS as t, EPG as e where (Status & 1) and NextEpgId > 0 and t.NextEpgId = e.eventid and not e.starttime between t.NextStartTime and t.NextStopTime";
    my $erg = $obj->{dbh}->selectall_hashref($sql, 'Id');

    foreach my $t (keys %$erg) {
        my $timer = $erg->{$t}->{Id};
        $erg->{Activ} = (($erg->{$t}->{Status} & 1) ? 'y' : 'n');
        $erg->{VPS} =   (($erg->{$t}->{Status} & 4) ? 'y' : 'n');
        $obj->saveTimer($erg, $timer);
    }
    return $erg;
}

# ------------------
sub getEpgIds {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $sql = "select Id, NextEpgId from TIMERS where NextEpgId > 0 and (Status & 1)";
    my $erg = $obj->{dbh}->selectall_hashref($sql, 'NextEpgId');
    return $erg;
}

# ------------------
sub getEpgDesc {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $tid = shift  || return error ('No TimerId!' );

    my $sql = qq|
select
    description from TIMERS as t, EPG as e
where
    NextEpgId > 0 and
    t.NextEpgId = e.eventid and
    t.id = $tid
|;

    my $erg = $obj->{dbh}->selectall_arrayref($sql);
    return $erg->[0][0];
}

# ------------------
sub getOverlappingTimer {
# ------------------
    my $obj  = shift || return error ('No Object!' );

    return unless(tableExists($obj->{dbh}, 'CHANNELS'));

    my $sql = qq|
select
    TIMERS.Id,
    TIMERS.Priority,
    TIMERS.NextStartTime,
    TIMERS.NextStopTime,
    CHANNELS.TID as transponderid,
    LEFT(CHANNELS.Source,1) as source
from TIMERS,
    CHANNELS
where TIMERS.ChannelID = CHANNELS.Id
|;
    my $erg = $obj->{dbh}->selectall_hashref($sql, 'Id');
    my $return;

    foreach my $tid (keys %$erg) {
        my $result = $obj->checkOverlapping($erg->{$tid});
        if(ref $result eq 'ARRAY' and scalar @$result) {
            $obj->{dbh}->do(sprintf('UPDATE TIMERS SET Collision = "%s" WHERE Id = %lu', join(',',@$result), $tid));
            $return->{"timer_$tid"} = join(',',@$result);
        }


    }
    return $return;
}

# ------------------
sub checkOverlapping {
# ------------------
    my $obj = shift  || return error ('No Object!' );
    my $data = shift  || return error ('No Data!' );

    my $NextStartTime =  $data->{NextStartTime};
    my $NextStopTime  =  $data->{NextStopTime};
    my $transponder   =  $data->{transponderid};
    my $source        =  $data->{source};
    my $Priority      =  $data->{Priority} || $obj->{Priority};
    my $tid           =  $data->{Id};

        my $sql = sprintf(
qq|
select
    t.Id,
    t.Priority,
    c.TID
from
    TIMERS as t, CHANNELS as c
where
    (('%s' between t.NextStartTime
        and t.NextStopTime)
        OR ('%s' between t.NextStartTime
        and t.NextStopTime)
    OR (t.NextStartTime between '%s'
        and '%s')
        OR (t.NextStopTime between '%s'
        and '%s'))
    and
        t.Id != %lu
    and
        (t.Status & 1)
    and
        t.ChannelID = c.Id
    and
        c.TID != %lu
    and
        LEFT(c.Source,1) = '%s'
    ORDER BY
        t.Priority desc
|,
    $NextStartTime,
    $NextStopTime,
    $NextStartTime,
    $NextStopTime,
    $NextStartTime,
    $NextStopTime,
    $tid,
    $transponder,
    $source
    );

    my $result = $obj->{dbh}->selectall_arrayref($sql);

    if(scalar @{$result}) {
            my $coltext = [];
            foreach my $probant (@{$result}) {

               if(defined $probant->[0]) {

               # current timer has higher Priority
               last
                  if($Priority > $probant->[1]);


               # Store conflict line at line
               my $col = sprintf('%d:%d',
                                    $probant->[0],
                                    $probant->[1]);

               # insert double transponder, on same line
               my $n = 0;
               foreach my $trans (@{$result}) {

                    if(defined $trans->[0]
                         && $probant->[0] != $trans->[0]
                         && $probant->[2] == $trans->[2]) {

                        $col .= sprintf('|%d:%d',
                                    $trans->[0],
                                    $trans->[1]);

                       undef @{$result}[$n]->[0];
                    }
                    ++$n;
               }
               # Add line
               push(@$coltext,$col);
           }
        }
        if(scalar(@$coltext) > $obj->{DVBCards} - 1) {
            return $coltext;
        }
    }
}

# ------------------
sub getNextTimer {
# ------------------
    my $obj = shift  || return error ('No Object!' );
    my $svdrp = main::getModule('SVDRP');

    my $erg = $svdrp->command('NEXT abs');
    my @eerg = grep(/^250/, @$erg);
    if(scalar @eerg and my ($errcode, $nextTimer, $zeit) = split(/\s+/, $eerg[0])) {
        return if(
            ! $nextTimer
            or $zeit < time
            or (ref $obj->{NextTimerEvent} and $obj->{NextTimerEvent}->at == $zeit)
        );

        my $timer = $obj->getTimerById($nextTimer);

        $obj->{NextTimerEvent} = Event->timer(
            at  => $zeit,
            data  => $timer,
            hard => 1,
            repeat => 0,
            prio => 2,  # -1 very hard ... 6 very low
            cb => sub{
                my $event = shift;
                my $watcher = $event->w;
                my $data = $watcher->data;

                my $reportmod = main::getModule('REPORT');
                $reportmod->news(
                    sprintf(gettext("Timer %d with title '%s' is start to recording!"), $data->{Id}, $data->{File}),
                    sprintf(gettext("on channel: %s until %s"), $data->{Channel}, fmttime($data->{Stop})),
                    'tedit',
                    $data->{Id},
                    'harmless'
                );
                $watcher->cancel;
            },
        );
    }
}


# ------------------
sub getNextEpgId {
# ------------------
    my $obj  = shift || return error ('No Object!' );

    return unless(tableExists($obj->{dbh}, 'EPG'));

    my $erg = getDataByTable('TIMERS');

    foreach my $tid (keys %$erg) {
        my $title = $erg->{$tid}->{File};
           $title =~ s/\'/\\\'/sg;
        my @file = split('~', $title);

        my $sql;
        if(scalar @file >= 2) { # title and subtitle defined
            $sql = sprintf(qq|
                select * from EPG
                WHERE
                    channel_id = '%s' AND
                    starttime between '%s' AND '%s' AND
                    (title like '%%%s%%' or title like '%%%s%%')
                |,
                $erg->{$tid}->{ChannelID},
                $erg->{$tid}->{NextStartTime},
                $erg->{$tid}->{NextStopTime},
                $file[-1],
                $file[-2]
            );
        } else {
                $sql = sprintf(qq|
                select * from EPG
                WHERE
                    channel_id = '%s' AND
                    starttime between '%s' AND '%s' AND
                    (title like '%%%s%%')
                |,
                $erg->{$tid}->{ChannelID},
                $erg->{$tid}->{NextStartTime},
                $erg->{$tid}->{NextStopTime},
                $title
            );
        }

        my $e = $obj->{dbh}->selectrow_hashref($sql);
        if(exists $e->{eventid}) {
            my $sql = sprintf('UPDATE TIMERS SET NextEpgId = %lu WHERE Id = %lu', $e->{eventid}, $tid);
            $obj->{dbh}->do($sql);
        }
    }
}

# The following subroutines is stolen from vdradmind and vdradmin-0.97-am
# Thanks on Cooper and Thomas for this great work!
# $obj->getNextTime('MDMDFSS', 1300, 1200)
# ------------------
sub getNextTime {
# ------------------
    my $obj = shift || return error ('No Object!' );
    my $dor = shift || return error ('No Day!' );
    my $start = shift || return error ('No Starttime!' );
    my $stop =  shift || return error ('No Stoptime!' );

    $start = sprintf('%04d', $start);
    $stop = sprintf('%04d', $stop);

    my ($startsse, $stopsse);
    if(length($dor) == 7) { # repeating timer => MTWTFSS
        $startsse = my_mktime(substr($start, 2, 2), substr($start, 0, 2),
                       my_strftime("%d"), (my_strftime("%m") - 1), my_strftime("%Y"));
        $stopsse  = my_mktime(substr($stop, 2, 2), substr($stop, 0, 2),
                       my_strftime("%d"), (my_strftime("%m") - 1), my_strftime("%Y"));
        $stopsse += 86400 if($stopsse < $startsse);

        my $weekday = ((localtime(time))[6] + 6) % 7;
        my $perrec = join("", substr($dor, $weekday), substr($dor, 0, $weekday));
        $perrec =~ m/^-+/g;

        my $off = (pos $perrec || 0) * 86400;
        if($off == 0 && $stopsse < time) {
            #$weekday = ($weekday + 1) % 7;
            $perrec = join("", substr($dor, ($weekday + 1) % 7), substr($dor, 0, ($weekday + 1) % 7));
            $perrec =~ m/^-+/g;
            $off = ((pos $perrec || 0) + 1) * 86400;
        }
        $startsse += $off;
        $stopsse += $off;
    } elsif(length($dor) == 18) { # first-day timer => MTWTFSS@2005-03-13
        $dor =~ /.{7}\@(\d{4})-(\d{2})-(\d{2})/;
        $startsse = my_mktime(substr($start, 2, 2),
                substr($start, 0, 2), $3, ($2 - 1), $1);
        # 31 + 1 = ??
        $stopsse = my_mktime(substr($stop, 2, 2),
                substr($stop, 0, 2), $stop > $start ? $3 : $3 + 1,
                ($2 - 1), $1);
    } else { # regular timer
      if ($dor =~ /(\d{4})-(\d{2})-(\d{2})/) { # vdr >= 1.3.23 => 2005-03-13
        $startsse = my_mktime(substr($start, 2, 2),
                              substr($start, 0, 2), $3, ($2 - 1), $1);

        $stopsse = my_mktime(substr($stop, 2, 2),
                             substr($stop, 0, 2), $stop > $start ? $3 : $3 + 1, ($2 - 1), $1);
        $obj->{newTimerFormat} = 1;
      }
      else { # vdr < 1.3.23 => 13
        $startsse = my_mktime(substr($start, 2, 2),
                substr($start, 0, 2), $dor, (my_strftime("%m") - 1),
                my_strftime("%Y"));

        $stopsse = my_mktime(substr($stop, 2, 2),
                substr($stop, 0, 2), $stop > $start ? $dor : $dor + 1,
                (my_strftime("%m") - 1), my_strftime("%Y"));

        # move timers which have expired one month into the future
        if(length($dor) != 7 && $stopsse < time) {
          $startsse = my_mktime(substr($start, 2, 2),
                                substr($start, 0, 2), $dor, (my_strftime("%m") % 12),
                                (my_strftime("%Y") + (my_strftime("%m") == 12 ? 1 : 0)));

          $stopsse = my_mktime(substr($stop, 2, 2),
                               substr($stop, 0, 2), $stop > $start ? $dor : $dor + 1,
                               (my_strftime("%m") % 12),
                               (my_strftime("%Y") + (my_strftime("%m") == 12 ? 1 : 0)));
        }
      }
    }

    my $ret = {
        start => $startsse,
        stop => $stopsse,
    };
    return $ret;
}

# ------------------
# Name:  getTimersByAutotimer
# Descr: Routine group Autotimer to Timers.
# Usage: $hash = $obj->getTimersByAutotimer([$aid, $aid, $aid, ...]);
# ------------------
sub getTimersByAutotimer {
    my $obj = shift  || return error ('No Object!' );
    my $aids = shift || return $obj->{AIDS};

    $obj->{AIDS} = {};
    for my $aid (@$aids) {
        $obj->{AIDS}->{$aid} = {
            allTimer => [],
            activeTimer => [],
            deactiveTimer => [],
        };
        my $erg = getDataBySearch('TIMERS', sprintf('AutotimerId = %d', $aid));
        map {
            my $type = ($_->[1] ? 'activeTimer' : 'deactiveTimer');
            push(@{$obj->{AIDS}->{$aid}->{$type}}, $_->[0]);
            push(@{$obj->{AIDS}->{$aid}->{allTimer}}, $_->[0]);
        } @$erg;
    }
    return $obj->{AIDS};
}

sub my_mktime {
    my $sec  = 0;
    my $min = shift;
    my $hour = shift;
    my $mday = shift;
    my $mon  = shift;
    my $year = shift() - 1900;

    return mktime($sec, $min, $hour, $mday, $mon, $year, 0, 0, -1);
}

sub my_strftime {
    my $format = shift;
    my $time = shift || time;
    return(strftime($format, localtime($time)));
}



1;
